function blkdiag(A, B)
    new_size = (size(A,1) + size(B,1), size(A,2) + size(B,2))
    res = zeros(eltype(A), new_size)
    res[1:size(A,1), 1:size(A,2)] .= A
    res[size(A,1)+1:end, size(A,2)+1:end] .= B
    return res
end

function matrix_emb(n::DirectPowerGroupElem, p::perm)
    Id = parent(n.elts[1])()
    elt = Diagonal([(-1)^(el == Id ? 0 : 1) for el in n.elts])
    elt = elt[:, p.d]
    return blkdiag(elt, elt)
end

function (g::WreathProductElem)(A::MatElem)
    g_inv = inv(g)
    G = matrix_emb(g.n, g_inv.p)
    G_inv = matrix_emb(g_inv.n, g.p)
    M = parent(A)
    return M(G)*A*M(G_inv)
end

function Base.:*(x::AbstractAlgebra.MatElem, P::Generic.perm)
   z = similar(x)
   m = nrows(x)
   n = ncols(x)
   for i = 1:m
      for j = 1:n
         z[i, j] = x[i,P[j]]
      end
   end
   return z
end

function (p::perm)(A::MatElem)
    length(p.d) == A.r == A.c || throw("Can't act via $p on matrix of size ($(A.r), $(A.c))")
    return p*A*inv(p)
end
